@@ -1,5 +1,5 @@
 /* Copyright (C) 2008-2020 Kentoku Shiba
-   Copyright (C) 2019-2020 MariaDB corp
+   Copyright (C) 2019, 2022, MariaDB Corporation.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 #include "hs_compat.h"
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#include "hstcpcli.hpp"
-#endif
 
 #define SPIDER_DBTON_SIZE 15
 
@@ -28,56 +25,31 @@
 #define SPIDER_DB_WRAPPER_MYSQL "mysql"
 #define SPIDER_DB_WRAPPER_MARIADB "mariadb"
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100204
 #define PLUGIN_VAR_CAN_MEMALLOC
 /*
-#define ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
 #define HASH_UPDATE_WITH_HASH_VALUE
 */
-#else
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-#define HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-#endif
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100002
 #define SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
 #define SPIDER_HAS_APPEND_FOR_SINGLE_QUOTE
 #define SPIDER_HAS_SHOW_SIMPLE_FUNC
 #define SPIDER_HAS_JT_HASH_INDEX_MERGE
 #define SPIDER_HAS_EXPR_CACHE_ITEM
-#else
-#define SPIDER_NEED_CHECK_CONDITION_AT_CHECKING_DIRECT_ORDER_LIMIT
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100007
 #define SPIDER_ITEM_HAS_CMP_TYPE
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100004
 #define SPIDER_HAS_TIME_STATUS
 #define SPIDER_HAS_DECIMAL_OPERATION_RESULTS_VALUE_TYPE
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100014
 #define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100100
 #define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY_AND_THDPTR
-#endif
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100108
 #define SPIDER_HAS_GROUP_BY_HANDLER
-#endif
 
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100200
 #define SPIDER_ORDER_HAS_ENUM_ORDER
-#endif
 
-#if defined(MARIADB_BASE_VERSION)
 #define SPIDER_ITEM_GEOFUNC_NAME_HAS_MBR
 #define SPIDER_HANDLER_AUTO_REPAIR_HAS_ERROR
-#endif
 
 class spider_db_conn;
 typedef spider_db_conn SPIDER_DB_CONN;
@@ -101,6 +73,8 @@ typedef st_spider_result SPIDER_RESULT;
 
 #define SPIDER_SQL_DOT_STR "."
 #define SPIDER_SQL_DOT_LEN (sizeof(SPIDER_SQL_DOT_STR) - 1)
+#define SPIDER_SQL_PERCENT_STR "%"
+#define SPIDER_SQL_PERCENT_LEN (sizeof(SPIDER_SQL_PERCENT_STR) - 1)
 #define SPIDER_SQL_HYPHEN_STR "-"
 #define SPIDER_SQL_HYPHEN_LEN (sizeof(SPIDER_SQL_HYPHEN_STR) - 1)
 
@@ -114,30 +88,9 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_TABLE_NAME_STR "`table_name`"
 #define SPIDER_SQL_TABLE_NAME_LEN sizeof(SPIDER_SQL_TABLE_NAME_STR) - 1
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#define SPIDER_SQL_HS_EQUAL_STR "="
-#define SPIDER_SQL_HS_EQUAL_LEN (sizeof(SPIDER_SQL_HS_EQUAL_STR) - 1)
-#define SPIDER_SQL_HS_GT_STR ">"
-#define SPIDER_SQL_HS_GT_LEN (sizeof(SPIDER_SQL_HS_GT_STR) - 1)
-#define SPIDER_SQL_HS_GTEQUAL_STR ">="
-#define SPIDER_SQL_HS_GTEQUAL_LEN (sizeof(SPIDER_SQL_HS_GTEQUAL_STR) - 1)
-#define SPIDER_SQL_HS_LT_STR "<"
-#define SPIDER_SQL_HS_LT_LEN (sizeof(SPIDER_SQL_HS_LT_STR) - 1)
-#define SPIDER_SQL_HS_INSERT_STR "+"
-#define SPIDER_SQL_HS_INSERT_LEN (sizeof(SPIDER_SQL_HS_INSERT_STR) - 1)
-#define SPIDER_SQL_HS_UPDATE_STR "U"
-#define SPIDER_SQL_HS_UPDATE_LEN (sizeof(SPIDER_SQL_HS_UPDATE_STR) - 1)
-#define SPIDER_SQL_HS_DELETE_STR "D"
-#define SPIDER_SQL_HS_DELETE_LEN (sizeof(SPIDER_SQL_HS_DELETE_STR) - 1)
-#define SPIDER_SQL_HS_INCREMENT_STR "+"
-#define SPIDER_SQL_HS_INCREMENT_LEN (sizeof(SPIDER_SQL_HS_INCREMENT_STR) - 1)
-#define SPIDER_SQL_HS_DECREMENT_STR "-"
-#define SPIDER_SQL_HS_DECREMENT_LEN (sizeof(SPIDER_SQL_HS_DECREMENT_STR) - 1)
-#endif
 #define SPIDER_SQL_HS_LTEQUAL_STR "<="
 #define SPIDER_SQL_HS_LTEQUAL_LEN (sizeof(SPIDER_SQL_HS_LTEQUAL_STR) - 1)
 
-#ifdef ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
 #define SPIDER_SQL_CASE_STR "case "
 #define SPIDER_SQL_CASE_LEN (sizeof(SPIDER_SQL_CASE_STR) - 1)
 #define SPIDER_SQL_WHEN_STR " when "
@@ -148,7 +101,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_ELSE_LEN (sizeof(SPIDER_SQL_ELSE_STR) - 1)
 #define SPIDER_SQL_END_STR " end"
 #define SPIDER_SQL_END_LEN (sizeof(SPIDER_SQL_END_STR) - 1)
-#endif
 
 #define SPIDER_SQL_USING_STR " using "
 #define SPIDER_SQL_USING_LEN (sizeof(SPIDER_SQL_USING_STR) - 1)
@@ -172,6 +124,8 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_IN_LEN (sizeof(SPIDER_SQL_IN_STR) - 1)
 #define SPIDER_SQL_NOT_IN_STR "not in("
 #define SPIDER_SQL_NOT_IN_LEN (sizeof(SPIDER_SQL_NOT_IN_STR) - 1)
+#define SPIDER_SQL_LIKE_STR " like "
+#define SPIDER_SQL_LIKE_LEN (sizeof(SPIDER_SQL_LIKE_STR) - 1)
 #define SPIDER_SQL_NOT_LIKE_STR "not like"
 #define SPIDER_SQL_NOT_LIKE_LEN (sizeof(SPIDER_SQL_NOT_LIKE_STR) - 1)
 #define SPIDER_SQL_AS_CHAR_STR " as char"
@@ -225,16 +179,9 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_LOP_CHK_PRM_PRF_LEN (sizeof(SPIDER_SQL_LOP_CHK_PRM_PRF_STR) - 1)
 
 #define SPIDER_CONN_KIND_MYSQL (1 << 0)
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#define SPIDER_CONN_KIND_HS_READ (1 << 2)
-#define SPIDER_CONN_KIND_HS_WRITE (1 << 3)
-#endif
 
 #define SPIDER_SQL_KIND_SQL (1 << 0)
 #define SPIDER_SQL_KIND_HANDLER (1 << 1)
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#define SPIDER_SQL_KIND_HS (1 << 2)
-#endif
 
 #define SPIDER_SQL_TYPE_SELECT_SQL (1 << 0)
 #define SPIDER_SQL_TYPE_INSERT_SQL (1 << 1)
@@ -245,11 +192,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_TYPE_DROP_TMP_TABLE_SQL (1 << 6)
 #define SPIDER_SQL_TYPE_OTHER_SQL (1 << 7)
 #define SPIDER_SQL_TYPE_HANDLER (1 << 8)
-#define SPIDER_SQL_TYPE_SELECT_HS (1 << 9)
-#define SPIDER_SQL_TYPE_INSERT_HS (1 << 10)
-#define SPIDER_SQL_TYPE_UPDATE_HS (1 << 11)
-#define SPIDER_SQL_TYPE_DELETE_HS (1 << 12)
-#define SPIDER_SQL_TYPE_OTHER_HS (1 << 13)
 
 enum spider_bulk_upd_start {
   SPD_BU_NOT_START,
@@ -455,12 +397,6 @@ public:
     IO_CACHE *file,
     uint32 arg_length
   );
-  bool append_with_prefill(
-    const char *s,
-    uint32 arg_length,
-    uint32 full_length,
-    char fill_char
-  );
   int strstr(
     const String &search,
     uint32 offset = 0
@@ -620,34 +556,26 @@ typedef struct spider_conn_holder
   spider_conn_holder *next;
 } SPIDER_CONN_HOLDER;
 
+/* Record information of a local (spider) table, for use of the spider
+group by handler. */
 typedef struct spider_table_holder
 {
   TABLE *table;
   ha_spider *spider;
+  /* alias of the table, in the form of tk, where k is the index of
+  the table from `query->from' indexed by next_local. */
   spider_string *alias;
 } SPIDER_TABLE_HOLDER;
 
-typedef struct spider_field_holder
-{
-  Field *field;
-  ha_spider *spider;
-  spider_string *alias;
-  spider_field_holder *next;
-} SPIDER_FIELD_HOLDER;
-
-typedef struct spider_field_chain
-{
-  spider_field_holder *field_holder;
-  spider_field_chain *next;
-} SPIDER_FIELD_CHAIN;
-
+/* For use of the spider group by handler. */
 class spider_fields
 {
   uint dbton_count;
   uint current_dbton_num;
   uint dbton_ids[SPIDER_DBTON_SIZE];
+  /* Number of tables in `query->from'. */
   uint table_count;
-  uint current_table_num;
+  /* All tables in `query->from', in the same order by next_local. */
   SPIDER_TABLE_HOLDER *table_holder;
   SPIDER_LINK_IDX_CHAIN *first_link_idx_chain;
   SPIDER_LINK_IDX_CHAIN *last_link_idx_chain;
@@ -656,13 +584,6 @@ class spider_fields
   SPIDER_CONN_HOLDER *first_conn_holder;
   SPIDER_CONN_HOLDER *last_conn_holder;
   SPIDER_CONN_HOLDER *current_conn_holder;
-  SPIDER_FIELD_HOLDER *first_field_holder;
-  SPIDER_FIELD_HOLDER *last_field_holder;
-  SPIDER_FIELD_HOLDER *current_field_holder;
-  SPIDER_FIELD_CHAIN *first_field_chain;
-  SPIDER_FIELD_CHAIN *last_field_chain;
-  SPIDER_FIELD_CHAIN *current_field_chain;
-  Field **first_field_ptr;
   Field **current_field_ptr;
 public:
   spider_fields();
@@ -705,8 +626,6 @@ public:
     long access_balance
   );
   SPIDER_CONN_HOLDER *create_conn_holder();
-  void set_pos_to_first_conn_holder();
-  SPIDER_CONN_HOLDER *get_next_conn_holder();
   bool has_conn_holder();
   void clear_conn_holder_from_conn();
   bool check_conn_same_conn(
@@ -720,24 +639,14 @@ public:
   void free_conn_holder(
     SPIDER_CONN_HOLDER *conn_holder_arg
   );
-  SPIDER_TABLE_HOLDER *add_table(
-    ha_spider *spider_arg
-  );
-  bool all_query_fields_are_query_table_members();
-  int create_table_holder(
+  SPIDER_TABLE_HOLDER *find_table(Field *field);
+  void set_table_holder(
+    SPIDER_TABLE_HOLDER *table_holder_arg,
     uint table_count_arg
   );
-  void set_pos_to_first_table_holder();
-  SPIDER_TABLE_HOLDER *get_next_table_holder();
+  SPIDER_TABLE_HOLDER *get_first_table_holder();
   SPIDER_TABLE_HOLDER *get_table_holder(TABLE *table);
   uint get_table_count();
-  int add_field(Field *field_arg);
-  SPIDER_FIELD_HOLDER *create_field_holder();
-  void set_pos_to_first_field_holder();
-  SPIDER_FIELD_HOLDER *get_next_field_holder();
-  SPIDER_FIELD_CHAIN *create_field_chain();
-  void set_pos_to_first_field_chain();
-  SPIDER_FIELD_CHAIN *get_next_field_chain();
   void set_field_ptr(Field **field_arg);
   Field **get_next_field_ptr();
   int ping_table_mon_from_table(
@@ -745,70 +654,6 @@ public:
   );
 };
 
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#define SPIDER_HS_UINT32_INFO dena::uint32_info
-#define SPIDER_HS_STRING_REF dena::string_ref
-#ifndef HANDLERSOCKET_MYSQL_UTIL
-#define SPIDER_HS_VECTOR std::vector
-class spider_db_hs_string_ref_buffer
-{
-  SPIDER_HS_VECTOR<SPIDER_HS_STRING_REF> hs_conds;
-public:
-  spider_db_hs_string_ref_buffer();
-  ~spider_db_hs_string_ref_buffer();
-  int init();
-  void clear();
-  int push_back(
-    SPIDER_HS_STRING_REF &cond
-  );
-  SPIDER_HS_STRING_REF *ptr();
-  uint size();
-};
-#else
-class spider_db_hs_string_ref_buffer
-{
-  bool                    hs_da_init;
-  DYNAMIC_ARRAY           hs_conds;
-  uint                    hs_conds_id;
-  const char              *hs_conds_func_name;
-  const char              *hs_conds_file_name;
-  ulong                   hs_conds_line_no;
-public:
-  spider_db_hs_string_ref_buffer();
-  ~spider_db_hs_string_ref_buffer();
-  int init();
-  void clear();
-  int push_back(
-    SPIDER_HS_STRING_REF &cond
-  );
-  SPIDER_HS_STRING_REF *ptr();
-  uint size();
-};
-#endif
-
-class spider_db_hs_str_buffer
-{
-  bool                    hs_da_init;
-  DYNAMIC_ARRAY           hs_conds;
-  uint                    hs_conds_id;
-  const char              *hs_conds_func_name;
-  const char              *hs_conds_file_name;
-  ulong                   hs_conds_line_no;
-public:
-  spider_db_hs_str_buffer();
-  ~spider_db_hs_str_buffer();
-  int init();
-  void clear();
-  spider_string *add(
-    uint *strs_pos,
-    const char *str,
-    uint str_len
-  );
-};
-
-#define SPIDER_DB_HS_STRING_REF_BUFFER spider_db_hs_string_ref_buffer
-#define SPIDER_DB_HS_STR_BUFFER spider_db_hs_str_buffer
-#endif
 
 struct st_spider_db_request_key
 {
@@ -823,8 +668,8 @@ class spider_db_util
 {
 public:
   uint dbton_id;
-  spider_db_util() {}
-  virtual ~spider_db_util() {}
+  spider_db_util() = default;
+  virtual ~spider_db_util() = default;
   virtual int append_name(
     spider_string *str,
     const char *name,
@@ -853,13 +698,10 @@ public:
   virtual int append_escaped_name_quote(
     spider_string *str
   ) = 0;
-  virtual int append_column_value(
-    ha_spider *spider,
-    spider_string *str,
-    Field *field,
-    const uchar *new_ptr,
-    CHARSET_INFO *access_charset
-  ) = 0;
+  virtual int append_column_value(ha_spider *spider, spider_string *str,
+                                  Field *field, const uchar *new_ptr,
+                                  bool is_like,
+                                  CHARSET_INFO *access_charset)= 0;
   virtual int append_trx_isolation(
     spider_string *str,
     int trx_isolation
@@ -946,11 +788,6 @@ public:
     TABLE_LIST *table_list,
     uint table_count
   ) = 0;
-  virtual int reappend_tables(
-    spider_fields *fields,
-    SPIDER_LINK_IDX_CHAIN *link_idx_chain,
-    spider_string *str
-  ) = 0;
   virtual int append_where(
     spider_string *str
   ) = 0;
@@ -971,7 +808,7 @@ public:
   uint dbton_id;
   SPIDER_DB_ROW *next_pos;
   spider_db_row(uint in_dbton_id) : dbton_id(in_dbton_id), next_pos(NULL) {}
-  virtual ~spider_db_row() {}
+  virtual ~spider_db_row() = default;
   virtual int store_to_field(
     Field *field,
     CHARSET_INFO *access_charset
@@ -1003,8 +840,8 @@ public:
 class spider_db_result_buffer
 {
 public:
-  spider_db_result_buffer() {}
-  virtual ~spider_db_result_buffer() {}
+  spider_db_result_buffer() = default;
+  virtual ~spider_db_result_buffer() = default;
   virtual void clear() = 0;
   virtual bool check_size(
     longlong size
@@ -1017,7 +854,7 @@ public:
   SPIDER_DB_CONN *db_conn;
   uint           dbton_id;
   spider_db_result(SPIDER_DB_CONN *in_db_conn);
-  virtual ~spider_db_result() {}
+  virtual ~spider_db_result() = default;
   virtual void set_limit(longlong value) {}
   virtual bool has_result() = 0;
   virtual void free_result() = 0;
@@ -1084,7 +921,7 @@ public:
   spider_db_conn(
     SPIDER_CONN *in_conn
   );
-  virtual ~spider_db_conn() {}
+  virtual ~spider_db_conn() = default;
   virtual int init() = 0;
   virtual void set_limit(longlong value) {}
   virtual bool is_connected() = 0;
@@ -1207,10 +1044,6 @@ public:
     Time_zone *time_zone,
     int *need_mon
   ) = 0;
-  virtual bool set_loop_check_in_bulk_sql();
-  virtual int set_loop_check(
-    int *need_mon
-  );
   virtual int fin_loop_check();
   virtual int show_master_status(
     SPIDER_TRX *trx,
@@ -1223,54 +1056,6 @@ public:
     SPIDER_DB_RESULT **res1,
     SPIDER_DB_RESULT **res2
   ) = 0;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  virtual int append_sql(
-    char *sql,
-    ulong sql_length,
-    st_spider_db_request_key *request_key
-  ) = 0;
-  virtual int append_open_handler(
-    uint handler_id,
-    const char *db_name,
-    const char *table_name,
-    const char *index_name,
-    const char *sql,
-    st_spider_db_request_key *request_key
-  ) = 0;
-  virtual int append_select(
-    uint handler_id,
-    spider_string *sql,
-    SPIDER_DB_HS_STRING_REF_BUFFER *keys,
-    int limit,
-    int skip,
-    st_spider_db_request_key *request_key
-  ) = 0;
-  virtual int append_insert(
-    uint handler_id,
-    SPIDER_DB_HS_STRING_REF_BUFFER *upds,
-    st_spider_db_request_key *request_key
-  ) = 0;
-  virtual int append_update(
-    uint handler_id,
-    spider_string *sql,
-    SPIDER_DB_HS_STRING_REF_BUFFER *keys,
-    SPIDER_DB_HS_STRING_REF_BUFFER *upds,
-    int limit,
-    int skip,
-    bool increment,
-    bool decrement,
-    st_spider_db_request_key *request_key
-  ) = 0;
-  virtual int append_delete(
-    uint handler_id,
-    spider_string *sql,
-    SPIDER_DB_HS_STRING_REF_BUFFER *keys,
-    int limit,
-    int skip,
-    st_spider_db_request_key *request_key
-  ) = 0;
-  virtual void reset_request_queue() = 0;
-#endif
   virtual size_t escape_string(
     char *to,
     const char *from,
@@ -1311,7 +1096,7 @@ public:
     st_spider_share *share,
     uint dbton_id
   ) : dbton_id(dbton_id), spider_share(share) {}
-  virtual ~spider_db_share() {}
+  virtual ~spider_db_share() = default;
   virtual int init() = 0;
   virtual uint get_column_name_length(
     uint field_index
@@ -1350,16 +1135,14 @@ public:
   uint dbton_id;
   ha_spider *spider;
   spider_db_share *db_share;
+  /* Index of active server, used in query construction. */
   int first_link_idx;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
-  SPIDER_LINK_IDX_CHAIN *link_idx_chain;
-#endif
-  bool strict_group_by;
-  bool no_where_cond;
+  bool strict_group_by= false;
+  bool no_where_cond= false;
   spider_db_handler(ha_spider *spider, spider_db_share *db_share) :
     dbton_id(db_share->dbton_id), spider(spider), db_share(db_share),
     first_link_idx(-1) {}
-  virtual ~spider_db_handler() {}
+  virtual ~spider_db_handler() = default;
   virtual int init() = 0;
   virtual int append_index_hint(
     spider_string *str,
@@ -1404,11 +1187,6 @@ public:
   virtual int append_insert_part() = 0;
   virtual int append_update_part() = 0;
   virtual int append_delete_part() = 0;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  virtual int append_increment_update_set_part() = 0;
-#endif
-#endif
   virtual int append_update_set_part() = 0;
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   virtual int append_direct_update_set_part() = 0;
@@ -1647,26 +1425,6 @@ public:
   virtual int reset_sql(
     ulong sql_type
   ) = 0;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  virtual int reset_keys(
-    ulong sql_type
-  ) = 0;
-  virtual int reset_upds(
-    ulong sql_type
-  ) = 0;
-  virtual int reset_strs(
-    ulong sql_type
-  ) = 0;
-  virtual int reset_strs_pos(
-    ulong sql_type
-  ) = 0;
-  virtual int push_back_upds(
-    SPIDER_HS_STRING_REF &info
-  ) = 0;
-#endif
-  virtual bool need_lock_before_set_sql_for_exec(
-    ulong sql_type
-  ) = 0;
 #ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int set_sql_for_exec(
     ulong sql_type,
@@ -1794,10 +1552,6 @@ public:
     spider_fields *fields,
     ulong sql_type
   ) = 0;
-  virtual int reappend_tables_part(
-    spider_fields *fields,
-    ulong sql_type
-  ) = 0;
   virtual int append_where_part(
     ulong sql_type
   ) = 0;
@@ -1858,7 +1612,7 @@ public:
   spider_db_share *db_share;
   spider_db_copy_table(spider_db_share *db_share) :
     dbton_id(db_share->dbton_id), db_share(db_share) {}
-  virtual ~spider_db_copy_table() {}
+  virtual ~spider_db_copy_table() = default;
   virtual int init() = 0;
   virtual void set_sql_charset(
     CHARSET_INFO *cs
@@ -1941,6 +1695,12 @@ static const LEX_CSTRING maturity_name[] =
   { STRING_WITH_LEN("Stable") }
 };
 
+/*
+  Type of singletons based on the type of the remote database.
+
+  All such singletons are stored in the array `spider_dbton', see
+  `spider_db_init()'.
+*/
 typedef struct st_spider_dbton
 {
   uint dbton_id;
@@ -2027,18 +1787,7 @@ typedef struct st_spider_result_list
     SPIDER_RESULT         *current;
   KEY                     *key_info;
   int                     key_order;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  ulonglong               hs_upd_rows;
-  SPIDER_DB_RESULT        *hs_result;
-  SPIDER_DB_RESULT_BUFFER *hs_result_buf;
-  bool                    hs_has_result;
-  SPIDER_DB_CONN          *hs_conn;
-#endif
 #ifdef HA_CAN_BULK_ACCESS
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uchar                   *hs_r_bulk_open_index;
-  uchar                   *hs_w_bulk_open_index;
-#endif
 #endif
   spider_string           *sqls;
   int                     ha_read_kind;
